@@ -59,6 +59,9 @@ __status__ = "Production"
 __docformat__ = 'reStructuredText'
 
 
+wav_type = "float32"  # can be int16, float64, int32 or float32
+
+
 def read_batch(batch_file):
     """
     :param batch_file:
@@ -307,23 +310,22 @@ class MFCC(object):
         :param sample:
         :return:
         """
+        sig = sample[0][:, numpy.newaxis] # ajout
+        framed = framing(sample[0], self.window_length, win_shift=self.window_length - self.overlap).copy()
         framed = framing(sample[0], self.window_length, win_shift=self.window_length - self.overlap).copy()
         # Pre-emphasis filtering is applied after framing to be consistent with stream processing
         framed = pre_emphasis(framed, self.prefac)
-
         # Windowing has been changed to hanning which is supposed to have less noisy sidelobes
         # ham = numpy.hamming(window_length)
         window = numpy.hanning(self.window_length)
         log_energy = numpy.log((framed ** 2).sum(axis=1))
         mag = numpy.fft.rfft(framed * window, self.n_fft, axis=-1)
         spec = mag.real ** 2 + mag.imag ** 2
-
         # Filter the spectrum through the triangle filter-bank
         mspec = numpy.log(numpy.dot(spec, self.fbank))  # A tester avec log10 et log
         # Use the DCT to 'compress' the coefficients (spectrum -> cepstrum domain)
         # The C0 term is removed as it is the constant term
         mfcc = dct(mspec, type=2, norm='ortho', axis=-1)[:, 1:self.nceps + 1]
-
         return mfcc.T, sample[1], sample[2], sample[3], sample[4], sample[5]
 
 
@@ -335,7 +337,8 @@ class SideSet(Dataset):
                  set_type="train",
                  chunk_per_segment=1,
                  overlap=0.,
-                 dataset_df=None
+                 dataset_df=None,
+                 min_duration=0.165
                  ):
         """
 
@@ -351,6 +354,7 @@ class SideSet(Dataset):
         self.sample_rate = int(dataset["sample_rate"])
         self.data_file_extension = dataset["data_file_extension"]
         self.transformation = ''
+        self.min_duration = min_duration
 
 
         if set_type == "train":
@@ -481,8 +485,10 @@ class SideSet(Dataset):
 
         sig, _ = soundfile.read(f"{self.data_path}/{self.sessions.iloc[index]['file_id']}{self.data_file_extension}",
                                 start=start_frame,
-                                stop=stop_frame
+                                stop=stop_frame,
+                                 dtype=wav_type
                                )
+        sig = sig.astype(numpy.float32)
         sig += 0.0001 * numpy.random.randn(sig.shape[0])
 
 
@@ -545,7 +551,14 @@ class IdMapSet(Dataset):
     DataSet that provide data according to a sidekit.IdMap object
     """
 
-    def __init__(self, idmap_name, data_root_path, file_extension, transform_pipeline=None):
+    def __init__(self,
+                 idmap_name,
+                 data_root_path,
+                 file_extension,
+                 transform_pipeline=None,
+                 frame_rate=100,
+                 min_duration=0.165
+                 ):
         """
 
         :param data_root_name:
@@ -560,6 +573,8 @@ class IdMapSet(Dataset):
         self.file_extension = file_extension
         self.len = self.idmap.leftids.shape[0]
         self.transform_pipeline = transform_pipeline
+        self.min_duration = min_duration
+        self.sample_rate = frame_rate
 
         _transform = []
         if transform_pipeline is not None:
@@ -583,15 +598,25 @@ class IdMapSet(Dataset):
             start = 0.0
 
         if self.idmap.start[index] is None and self.idmap.stop[index] is None:
-            sig, _ = soundfile.read(f"{self.data_root_path}/{self.idmap.rightids[index]}.{self.file_extension}")
+            sig, sample_rate = soundfile.read(f"{self.data_root_path}/{self.idmap.rightids[index]}.{self.file_extension}", dtype=wav_type)
+            sig = sig.astype(numpy.float32)
             start = 0
             stop = len(sig)
         else:
+            nfo = soundfile.info(f"{self.data_root_path}/{self.idmap.rightids[index]}.{self.file_extension}")
             start = int(self.idmap.start[index])
             stop = int(self.idmap.stop[index])
+            # add this in case the segment is too short
+            if stop - start <= self.min_duration * nfo.samplerate:
+                middle = start + (stop - start) // 2
+                start = max(0, int(middle - (self.min_duration * nfo.samplerate / 2)))
+
+                stop = int(start + self.min_duration * nfo.samplerate)
             sig, _ = soundfile.read(f"{self.data_root_path}/{self.idmap.rightids[index]}.{self.file_extension}",
                                     start=start,
-                                    stop=stop)
+                                    stop=stop,
+                                    dtype=wav_type)
+            sig = sig.astype(numpy.float32)
         sig += 0.0001 * numpy.random.randn(sig.shape[0])
 
         if self.transform_pipeline is not None:
