@@ -30,6 +30,7 @@ import pdb
 import traceback
 import logging
 import matplotlib.pyplot as plt
+import multiprocessing
 import numpy
 import pandas
 import pickle
@@ -134,6 +135,7 @@ def speech_to_probs(model, speech):
 
 def plot_classes_preds(model, speech, labels):
     '''
+
     Generates matplotlib Figure using a trained network, along with images
     and labels from a batch, that shows the network's top prediction along
     with its probability, alongside the actual label, coloring this
@@ -271,19 +273,19 @@ class Xtractor(torch.nn.Module):
             self.sequence_network = torch.nn.Sequential(OrderedDict([
                 ("conv1", torch.nn.Conv1d(self.feature_size, 512, 5, dilation=1)),
                 ("activation1", torch.nn.LeakyReLU(0.2)),
-                ("norm1", torch.nn.BatchNorm1d(512)),
+                ("batch_norm1", torch.nn.BatchNorm1d(512)),
                 ("conv2", torch.nn.Conv1d(512, 512, 3, dilation=2)),
                 ("activation2", torch.nn.LeakyReLU(0.2)),
-                ("norm2", torch.nn.BatchNorm1d(512)),
+                ("batch_norm2", torch.nn.BatchNorm1d(512)),
                 ("conv3", torch.nn.Conv1d(512, 512, 3, dilation=3)),
                 ("activation3", torch.nn.LeakyReLU(0.2)),
-                ("norm3", torch.nn.BatchNorm1d(512)),
+                ("batch_norm3", torch.nn.BatchNorm1d(512)),
                 ("conv4", torch.nn.Conv1d(512, 512, 1)),
                 ("activation4", torch.nn.LeakyReLU(0.2)),
-                ("norm4", torch.nn.BatchNorm1d(512)),
+                ("batch_norm4", torch.nn.BatchNorm1d(512)),
                 ("conv5", torch.nn.Conv1d(512, 1536, 1)),
                 ("activation5", torch.nn.LeakyReLU(0.2)),
-                ("norm5", torch.nn.BatchNorm1d(1536))
+                ("batch_norm5", torch.nn.BatchNorm1d(1536))
             ]))
 
             self.stat_pooling = MeanStdPooling()
@@ -299,11 +301,11 @@ class Xtractor(torch.nn.Module):
             elif self.loss == "cce":
                 self.after_speaker_embedding = torch.nn.Sequential(OrderedDict([
                     ("activation6", torch.nn.LeakyReLU(0.2)),
-                    ("norm6", torch.nn.BatchNorm1d(512)),
+                    ("batch_norm6", torch.nn.BatchNorm1d(512)),
                     ("dropout6", torch.nn.Dropout(p=0.05)),
                     ("linear7", torch.nn.Linear(512, 512)),
                     ("activation7", torch.nn.LeakyReLU(0.2)),
-                    ("norm7", torch.nn.BatchNorm1d(512)),
+                    ("batch_norm7", torch.nn.BatchNorm1d(512)),
                     ("linear8", torch.nn.Linear(512, int(self.speaker_number)))
                 ]))
 
@@ -359,14 +361,17 @@ class Xtractor(torch.nn.Module):
             self.after_speaker_embedding_weight_decay = 0.00
 
         else:
-            # Load Yaml configuration
-            with open(model_archi, 'r') as fh:
-                cfg = yaml.load(fh, Loader=yaml.FullLoader)
+            if isinstance(model_archi, dict):
+                cfg = model_archi
+            else:
+                # Load Yaml configuration
+                with open(model_archi, 'r') as fh:
+                    cfg = yaml.load(fh, Loader=yaml.FullLoader)
 
-            self.loss = cfg["loss"]
+            self.loss = cfg["training"]["loss"]
             if self.loss == "aam":
-                self.aam_margin = cfg["aam_margin"]
-                self.aam_s = cfg["aam_s"]
+                self.aam_margin = cfg["training"]["aam_margin"]
+                self.aam_s = cfg["training"]["aam_s"]
 
             """
             Prepare Preprocessor
@@ -397,6 +402,7 @@ class Xtractor(torch.nn.Module):
                                                         padding=cfg['preprocessor']["padding"],
                                                         dilation=cfg['preprocessor']["dilation"])
                     self.feature_size = cfg["feature_size"]
+                self.preprocessor_weight_decay = 0.000
 
             """
             Prepare sequence network
@@ -445,10 +451,12 @@ class Xtractor(torch.nn.Module):
             Pooling
             """
             self.stat_pooling = MeanStdPooling()
+            tmp_input_size = input_size * 2
             if cfg["stat_pooling"]["type"] == "GRU":
                 self.stat_pooling = GruPooling(input_size=cfg["stat_pooling"]["input_size"],
                                                gru_node=cfg["stat_pooling"]["gru_node"],
                                                nb_gru_layer=cfg["stat_pooling"]["nb_gru_layer"])
+                tmp_input_size = cfg["stat_pooling"]["gru_node"]
 
             self.stat_pooling_weight_decay = cfg["stat_pooling"]["weight_decay"]
 
@@ -456,7 +464,7 @@ class Xtractor(torch.nn.Module):
             Prepare last part of the network (after pooling)
             """
             # Create sequential object for the second part of the network
-            input_size = input_size * 2
+            input_size = tmp_input_size
             before_embedding_layers = []
             for k in cfg["before_embedding"].keys():
                 if k.startswith("lin"):
@@ -510,6 +518,7 @@ class Xtractor(torch.nn.Module):
                 self.after_speaker_embedding = torch.nn.Sequential(OrderedDict(after_embedding_layers))
 
             elif self.loss == "aam":
+                self.norm_embedding = True
                 self.after_speaker_embedding = ArcLinear(input_size,
                                                          self.speaker_number,
                                                          margin=self.aam_margin,
@@ -527,7 +536,6 @@ class Xtractor(torch.nn.Module):
         """
         if self.preprocessor is not None:
             x = self.preprocessor(x)
-            print("go through preprocessor")
 
         x = self.sequence_network(x)
 
@@ -537,7 +545,7 @@ class Xtractor(torch.nn.Module):
         x = self.before_speaker_embedding(x)
 
         if self.norm_embedding:
-            x_norm = x.norm(p=2,dim=1, keepdim=True) / 10.
+            x_norm = x.norm(p=2,dim=1, keepdim=True) / 10. # Why  10. ?
             x = torch.div(x, x_norm)
 
         if is_eval:
@@ -554,25 +562,36 @@ class Xtractor(torch.nn.Module):
 
         return x
 
+    def context_size(self):
+        context = 1
+        if isinstance(self, Xtractor):
+            for name, module in self.sequence_network.named_modules():
+                if name.startswith("conv"):
+                    context += module.dilation[0] * (module.kernel_size[0] - 1)
+        else:
+            for name, module in self.module.sequence_network.named_modules():
+                if name.startswith("conv"):
+                    context += module.dilation[0] * (module.kernel_size[0] - 1)
+        return context
 
 def xtrain(speaker_number,
            dataset_yaml,
-           epochs=100,
-           lr=0.01,
+           epochs=None,
+           lr=None,
            model_yaml=None,
            model_name=None,
-           loss="cce",
-           aam_margin=0.5,
-           aam_s=30,
-           patience=10,
+           loss=None,
+           aam_margin=None,
+           aam_s=None,
+           patience=None,
            tmp_model_name=None,
            best_model_name=None,
            multi_gpu=True,
            clipping=False,
-           opt='sgd',
+           opt=None,
            reset_parts=[],
            freeze_parts=[],
-           num_thread=1):
+           num_thread=None):
     """
 
     :param speaker_number:
@@ -600,43 +619,74 @@ def xtrain(speaker_number,
     #writer = SummaryWriter("runs/xvectors_experiments_2")
     writer = None
 
+    if num_thread is None:
+        num_thread = multiprocessing.cpu_count()
+
     logging.critical(f"Start process at {time.strftime('%H:%M:%S', time.localtime())}")
 
     device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
 
     # Start from scratch
-    if model_name is None:
+    if model_name is None and model_yaml in ["xvector", "rawnet2"]:
         # Initialize a first model
         if model_yaml == "xvector":
             model = Xtractor(speaker_number, "xvector")
         elif model_yaml == "rawnet2":
             model = Xtractor(speaker_number, "rawnet2")
-        else:
-            model = Xtractor(speaker_number, model_yaml)
-    # If we start from an existing model
     else:
-        # Load the model
-        logging.critical(f"*** Load model from = {model_name}")
-        checkpoint = torch.load(model_name)
-        model = Xtractor(speaker_number, model_yaml)
+        with open(model_yaml, 'r') as fh:
+            model_archi = yaml.load(fh, Loader=yaml.FullLoader)
+            if epochs is None:
+                epochs = model_archi["training"]["epochs"]
+            if patience is None:
+                patience = model_archi["training"]["patience"]
+            if opt is None:
+                opt = model_archi["training"]["opt"]
+            if lr is None:
+                lr = model_archi["training"]["lr"]
+            if loss is None:
+                loss = model_archi["training"]["loss"]
+            if aam_margin is None and model_archi["training"]["loss"] == "aam":
+                aam_margin = model_archi["training"]["aam_margin"]
+            if aam_s is None and model_archi["training"]["loss"] == "aam":
+                aam_s = model_archi["training"]["aam_s"]
+            if tmp_model_name is None:
+                tmp_model_name = model_archi["training"]["tmp_model_name"]
+            if best_model_name is None:
+                best_model_name = model_archi["training"]["best_model_name"]
+            if multi_gpu is None:
+                multi_gpu = model_archi["training"]["multi_gpu"]
+            if clipping is None:
+                clipping = model_archi["training"]["clipping"]
 
-        """
-        Here we remove all layers that we don't want to reload
+        if model_name is None:
+            model = Xtractor(speaker_number, model_yaml)
+
+         # If we start from an existing model
+        else:
+            # Load the model
+            logging.critical(f"*** Load model from = {model_name}")
+            checkpoint = torch.load(model_name)
+            model = Xtractor(speaker_number, model_yaml)
+
+            """
+            Here we remove all layers that we don't want to reload
         
-        """
-        pretrained_dict = checkpoint["model_state_dict"]
-        for part in reset_parts:
-            pretrained_dict = {k: v for k, v in pretrained_dict.items() if not k.startswith(part)}
+            """
+            pretrained_dict = checkpoint["model_state_dict"]
+            for part in reset_parts:
+                pretrained_dict = {k: v for k, v in pretrained_dict.items() if not k.startswith(part)}
 
-        new_model_dict = model.state_dict()
-        new_model_dict.update(pretrained_dict)
-        model.load_state_dict(new_model_dict)
+            new_model_dict = model.state_dict()
+            new_model_dict.update(pretrained_dict)
+            model.load_state_dict(new_model_dict)
 
-    # Freeze required layers
-    for name, param in model.named_parameters():
-        if name.split(".")[0] in freeze_parts:
-            param.requires_grad = False
+        # Freeze required layers
+        for name, param in model.named_parameters():
+            if name.split(".")[0] in freeze_parts:
+                param.requires_grad = False
 
+    print(model)
 
     if torch.cuda.device_count() > 1 and multi_gpu:
         print("Let's use", torch.cuda.device_count(), "GPUs!")
@@ -655,7 +705,6 @@ def xtrain(speaker_number,
         dataset_params = yaml.load(fh, Loader=yaml.FullLoader)
         df = pandas.read_csv(dataset_params["dataset_description"])
     training_df, validation_df = train_test_split(df, test_size=dataset_params["validation_ratio"])
-
     torch.manual_seed(dataset_params['seed'])
     training_set = SideSet(dataset_yaml, 
                            set_type="train", 
@@ -771,7 +820,9 @@ def xtrain(speaker_number,
                 'model_state_dict': model.state_dict(),
                 'optimizer_state_dict': optimizer.state_dict(),
                 'accuracy': best_accuracy,
-                'scheduler': scheduler
+                'scheduler': scheduler,
+                'speaker_number' : speaker_number,
+                'model_archi': model_archi
             }, is_best, filename=tmp_model_name+".pt", best_filename=best_model_name+'.pt')
         else:
             save_checkpoint({
@@ -779,7 +830,9 @@ def xtrain(speaker_number,
                 'model_state_dict': model.module.state_dict(),
                 'optimizer_state_dict': optimizer.state_dict(),
                 'accuracy': best_accuracy,
-                'scheduler': scheduler
+                'scheduler': scheduler,
+                'speaker_number': speaker_number,
+                'model_archi': model_archi
             }, is_best, filename=tmp_model_name+".pt", best_filename=best_model_name+'.pt')
 
         if is_best:
@@ -909,26 +962,43 @@ def cross_validation(model, validation_loader, device):
 
 
 def extract_embeddings(idmap_name,
-                       speaker_number,
                        model_filename,
-                       model_yaml,
-                       data_root_name ,
+                       data_root_name,
                        device,
+                       model_yaml=None,
+                       speaker_number=None,
                        file_extension="wav",
                        transform_pipeline=None,
+                       frame_shift=0.01,
+                       frame_duration=0.025,
                        num_thread=1):
+    # Load the model
+    if isinstance(model_filename, str):
+        checkpoint = torch.load(model_filename)
+        if speaker_number is None:
+            speaker_number = checkpoint["speaker_number"]
+        if model_yaml is None:
+            model_archi = checkpoint["model_archi"]
+        model = Xtractor(speaker_number, model_archi=model_archi)
+        model.load_state_dict(checkpoint["model_state_dict"])
+    else:
+        model = model_filename
 
     if isinstance(idmap_name, IdMap):
         idmap = idmap_name
     else:
         idmap = IdMap(idmap_name)
 
+    min_duration = (model.context_size() - 1) * frame_shift + frame_duration
+
     # Create dataset to load the data
     dataset = IdMapSet(idmap_name=idmap_name,
                        data_root_path=data_root_name,
                        file_extension=file_extension,
-                       transform_pipeline=transform_pipeline)
-
+                       transform_pipeline=transform_pipeline,
+                       frame_rate=int(1. / frame_shift),
+                       min_duration=(model.context_size() + 2) * frame_shift * 2
+                       )
 
     dataloader = DataLoader(dataset,
                             batch_size=1,
@@ -938,13 +1008,6 @@ def extract_embeddings(idmap_name,
                             num_workers=num_thread)
 
     with torch.no_grad():
-        # Load the model
-        if isinstance(model_filename, str):
-            checkpoint = torch.load(model_filename)
-            model = Xtractor(speaker_number, model_archi=model_yaml)
-            model.load_state_dict(checkpoint["model_state_dict"])
-        else:
-            model = model_filename
 
         model.eval()
         model.to(device)
@@ -964,16 +1027,8 @@ def extract_embeddings(idmap_name,
 
         # Process the data
         with torch.no_grad():
-            #for idx in tqdm.tqdm(range(len(dataset))):
             for idx, (data, mod, seg, start, stop) in tqdm.tqdm(enumerate(dataloader)):
-                #data, mod, seg, start, stop = dataset[idx]
                 vec = model(data.to(device), is_eval=True)
-                #vec = model(data[None, :, :].to(device), is_eval=True)
-                #current_idx = numpy.argwhere(numpy.logical_and(idmap.leftids == mod, idmap.rightids == seg))[0][0]
-                #embeddings.start[idx] = start
-                #embeddings.stop[idx] = stop
-                #embeddings.modelset[idx] = mod
-                #embeddings.segset[idx] = seg
                 embeddings.stat1[idx, :] = vec.detach().cpu()
 
     return embeddings
